/*
 * Windows Midi API for SWIG/python
 * Copyright David Leonard, 2004.
 */

%module winmm_midi
%include "typemaps.i"
%include "exception.i"
%include "cstring.i"

%{
#include <windows.h>
#include <mmsystem.h>
%}

/* from <windef.h> */
typedef unsigned char  CHAR;		/* 8 bit */
typedef unsigned short WORD;		/* 16 bit */
typedef unsigned long  DWORD;		/* 32 bit */
typedef unsigned int   UINT;
typedef void *         HANDLE;
typedef bool	       BOOL;

typedef UINT MMRESULT;
typedef UINT MMVERSION;

/* An opaque handle to an opened MIDI input. */
typedef HANDLE HMIDIIN;

/* values for wTechnology */
%constant WORD MOD_MIDIPORT;
%constant WORD MOD_SYNTH;
%constant WORD MOD_SQSYNTH;
%constant WORD MOD_FMSYNTH;
%constant WORD MOD_MAPPER;
//#ifdef MOD_WAVETABLE
//%constant WORD MOD_WAVETABLE;
//#endif
//#ifdef MOD_SWSYNTH
//%constant WORD MOD_SWSYNTH;
//#endif

/* values for dwSupport */
%constant DWORD MIDICAPS_CACHE;
%constant DWORD MIDICAPS_LRVOLUME;
%constant DWORD MIDICAPS_STREAM;
%constant DWORD MIDICAPS_VOLUME;

typedef HANDLE HMIDIOUT;

%rename(midiInGetDevCaps) XmidiInGetDevCaps;
%rename(midiOutGetDevCaps) XmidiOutGetDevCaps;
PyObject *XmidiInGetDevCaps(UINT id);
PyObject *XmidiOutGetDevCaps(UINT id);

typedef struct MIDIHDR {
    %extend {
	MIDIHDR(DWORD);
	~MIDIHDR();
	void reset();
	void append(PyObject *);
        %immutable;
	DWORD	dwBytesRecorded;
    }
	PyObject *dwUser;
	DWORD	dwFlags;
	DWORD	dwOffset;
} MIDIHDR;

/* values for dwFlags */
%constant DWORD MHDR_DONE;
%constant DWORD MHDR_INQUEUE;
%constant DWORD MHDR_ISSTRM;
%constant DWORD MHDR_PREPARED;

%typemap(default) UINT midiHdrSz "$1 = sizeof (MIDIHDR);"
MMRESULT midiInPrepareHeader(HMIDIIN midiIn, MIDIHDR *hdr, UINT midiHdrSz);
MMRESULT midiInUnprepareHeader(HMIDIIN midiIn, MIDIHDR *hdr, UINT midiHdrSz);
MMRESULT midiInAddBuffer(HMIDIIN midiIn, MIDIHDR *hdr, UINT midiHdrSz);
MMRESULT midiOutPrepareHeader(HMIDIOUT midiOut, MIDIHDR *hdr, UINT midiHdrSz);
MMRESULT midiOutUnprepareHeader(HMIDIOUT midiOut, MIDIHDR *hdr,UINT midiHdrSz);

%constant int MAXERRORLENGTH;
%cstring_output_maxsize(char *textstr, int textlen);
MMRESULT midiInGetErrorText(MMRESULT error, char *textstr, int textlen = MAXERRORLENGTH);
MMRESULT midiOutGetErrorText(MMRESULT error, char *textstr, int textlen);

UINT midiInGetNumDevs(void);
MMRESULT midiInGetID(HMIDIIN midiIn, UINT *OUTPUT);
MMRESULT midiInReset(HMIDIIN midiIn);
MMRESULT midiInStart(HMIDIIN midiIn);
MMRESULT midiInStop(HMIDIIN midiIn);

UINT midiOutGetNumDevs(void);
MMRESULT midiOutGetID(HMIDIOUT midiOut, UINT *OUTPUT);
MMRESULT midiOutGetVolume(HMIDIOUT midiOut, DWORD *OUTPUT);
MMRESULT midiOutReset(HMIDIOUT midiOut);
MMRESULT midiOutSetVolume(HMIDIOUT midiOut, DWORD volume);
MMRESULT midiOutLongMsg(HMIDIOUT midiOut, MIDIHDR *hdr, UINT midiHdrSz);

%rename(midiOutShortMsg) XmidiOutShortMsg;
MMRESULT XmidiOutShortMsg(HMIDIOUT midiOut, UINT msg, UINT parm1 = 0, UINT parm2 = 0);

/*
 * When opening a MIDI channel, the system can call a C callback function.
 * Because these functions are very limited in what they can do, we provide
 * a circular queue with a Win32 event handle. The callback function adds
 * messages into the circular queue and signals the event handle. A
 * Python read() method on the queue returns the stored words.
 */

typedef struct MidiMsgQueue {
    %extend {
	    MidiMsgQueue(UINT size);
	    ~MidiMsgQueue();
	    PyObject *read(DWORD timeout);
	    long get_event_handle();
    }
    UINT overflowed;
    UINT recycle;
} MidiMsgQueue;
%constant DWORD INFINITE;
%constant DWORD MIM_DATA;
%constant DWORD MIM_ERROR;
%constant DWORD MIM_LONGDATA;
%constant DWORD MIM_LONGERROR;
%constant DWORD MIM_MOREDATA;

%rename(midiInOpen) XmidiInOpen;
%rename(midiOutOpen) XmidiOutOpen;
HMIDIIN XmidiInOpen(MMRESULT *OUTPUT, UINT id, MidiMsgQueue *queue = NULL);
HMIDIOUT XmidiOutOpen(MMRESULT *OUTPUT, UINT id);

MMRESULT midiInClose(HMIDIIN midiIn);
MMRESULT midiOutClose(HMIDIOUT midiOut);

%constant MMRESULT MMSYSERR_ERROR;
%constant MMRESULT MMSYSERR_BADDEVICEID;
%constant MMRESULT MMSYSERR_NOTENABLED;
%constant MMRESULT MMSYSERR_ALLOCATED;
%constant MMRESULT MMSYSERR_INVALHANDLE;
%constant MMRESULT MMSYSERR_NODRIVER;
%constant MMRESULT MMSYSERR_NOMEM;
%constant MMRESULT MMSYSERR_NOTSUPPORTED;
%constant MMRESULT MMSYSERR_BADERRNUM;
%constant MMRESULT MMSYSERR_INVALFLAG;
%constant MMRESULT MMSYSERR_INVALPARAM;
%constant MMRESULT MMSYSERR_HANDLEBUSY;
%constant MMRESULT MMSYSERR_INVALIDALIAS;
%constant MMRESULT MMSYSERR_BADDB;
%constant MMRESULT MMSYSERR_KEYNOTFOUND;
%constant MMRESULT MMSYSERR_READERROR;
%constant MMRESULT MMSYSERR_WRITEERROR;
%constant MMRESULT MMSYSERR_DELETEERROR;
%constant MMRESULT MMSYSERR_VALNOTFOUND;
%constant MMRESULT MMSYSERR_NODRIVERCB;
%constant MMRESULT MMSYSERR_LASTERROR;

%{
#include "winmm_midi_impl.c"
%}
